﻿using System.Threading;

using Microsoft.AspNet.SignalR;

namespace MyProject.SignalR
{
    /// <summary>
    /// Proxy class for sending SignalR messages to Google Chrome.
    /// </summary>
    internal sealed class ChromeProxy
    {
        private static readonly ChromeProxy instance = new ChromeProxy();

        private object lockObj = new object();
        private string[] urls;
        private ManualResetEvent ev = new ManualResetEvent(false);

        // Explicit static constructor to tell C# compiler
        // not to mark type as beforefieldinit
        static ChromeProxy()
        {
        }

        /// <summary>
        /// Returns the instance of the singleton.
        /// </summary>
        public static ChromeProxy Instance
        {
            get { return instance; }
        }

        /// <summary>
        /// Gets or sets an array with URLs.
        /// </summary>
        public string[] URLs
        {
            get { return urls; }
            set
            {
                urls = value;
                ev.Set();
            }
        }

        /// <summary>
        /// Sends a SignalR message to Google Chrome and wait up to 3 seconds for Chrome to
        /// respond with a list if URLs.
        /// </summary>
        /// <returns>array with URLs</returns>
        public string[] GetBrowserUrls()
        {
            if (SignalRHub.ConnectedClients > 0)
            {
                lock (lockObj)
                {
                    ev.Reset();

                    // request URLs, Chrome will invoke SignalRHub.GetUrls()
                    IHubContext context = GlobalHost.ConnectionManager.GetHubContext<SignalRHub>();
                    if (context != null)
                    {
                        context.Clients.All.invokeGetUrls();

                        // wait up to 3 seconds for Chrome to respond
                        if (ev.WaitOne(3000, false))
                        {
                            return URLs;
                        }
                    }
                }
            }
            return new string[0];
        }

        /// <summary>
        /// Send a SignalR message to Google Chrome to open an URL in an existing tab.
        /// </summary>
        /// <param name="newUrl">URL to open</param>
        /// <param name="existingUrl">URL of existing tab</param>
        public void OpenUrl(string newUrl, string existingUrl)
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<SignalRHub>();
            if (context != null)
            {
                context.Clients.All.setUrl(newUrl, existingUrl);
            }
        }


        /// <summary>
        /// Disconnects all clients so they go to reconnect mode again.
        /// </summary>
        public void DisconnectClients()
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<SignalRHub>();
            if (context != null)
            {
                context.Clients.All.disconnect();
            }
        }
    }
}
