﻿using System;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.AspNet.SignalR;

namespace MyProject.SignalR
{
    /// <summary>
    /// This SignalR hub class is created as soon as a client (Chrome browser) connects to the server.
    /// It contains methods that are called by the client.
    /// </summary>
    public class SignalRHub : Hub
    {
        private static int clientCount = 0;

        /// <summary>
        /// Chrome registers itself.
        /// </summary>
        public void Register()
        {
            Clients.Caller.registerOk();
            Console.WriteLine("Client successfully registered to server.");
        }

        /// <summary>
        /// Chrome sends an array with all open URLs.
        /// </summary>
        /// <param name="tabUrls">array of URLs</param>
        public void GetUrls(string[] tabUrls)
        {
            Console.WriteLine("URLs received from browser:");
            foreach (string tabUrl in tabUrls)
            {
                Console.WriteLine("  " + tabUrl);
            }

            ChromeProxy.Instance.URLs = tabUrls;
        }

        /// <summary>
        /// Chrome asks for the phone number format.
        /// </summary>
        public void GetPhoneNumberFormat()
        {
            Console.WriteLine("Phone number format requested.");
            Clients.Caller.setFormat("123456");
        }

        /// <summary>
        /// Chrome asks for the phone number replacement.
        /// </summary>
        public void GetPhoneNumberReplacement()
        {
            Console.WriteLine("Phone number replacement requested.");
            Clients.Caller.setReplacement("$1$2$3$4");
        }

        /// <summary>
        /// Chrome asks for the protocol.
        /// </summary>
        public void GetProtocol()
        {
            Console.WriteLine("Protocol requested.");
            Clients.Caller.setProtocol("tel");
        }

        /// <summary>
        /// Increment the number of connected clients.
        /// </summary>
        public override Task OnConnected()
        {
            Interlocked.Increment(ref clientCount);
            return base.OnConnected();
        }

        /// <summary>
        /// Increment the number of connected clients.
        /// </summary>
        public override Task OnReconnected()
        {
            Interlocked.Increment(ref clientCount);
            return base.OnReconnected();
        }

        public override Task OnDisconnected(bool stopCalled)
        {
            Interlocked.Decrement(ref clientCount);
            return base.OnDisconnected(stopCalled);
        }

        /// <summary>
        /// Gets the number of connected clients.
        /// </summary>
        public static int ConnectedClients
        {
            get { return clientCount; }
        }
    }
}
